@@ -10,7 +10,8 @@ import slow
 # functions to the top level, but move on if not successful.
 try:
     from func import (nansum, nanmax, nanmin, nanmean, nanstd, nanvar, median,
-                      nanmedian, nanargmin, nanargmax)
+                      nanmedian, nanargmin, nanargmax, rankdata, nanrankdata,
+                      ss, partsort, argpartsort)
 except:
     pass
 try:
@@ -18,11 +19,8 @@ try:
                       move_mean, move_nanmean,
                       move_std, move_nanstd,
                       move_min, move_nanmin,
-                      move_max, move_nanmax)
-except:
-    pass
-try:
-    from group import group_nanmean
+                      move_max, move_nanmax,
+                      move_median)
 except:
     pass
 
