@@ -1,16 +1,53 @@
 
+# For support of python 2.5
+from __future__ import with_statement
+
 import numpy as np
-try:
-    import scipy as sp
-    SCIPY = True
-except ImportError:
-    SCIPY = False
 import bottleneck as bn
 from autotimeit import autotimeit
 
 __all__ = ['bench']
 
-def bench(mode='fast', dtype='float64', axis=0):
+
+def bench(mode='fast', dtype='float64', axis=0,
+          shapes=[(10,10),(100,100),(1000,1000),(10,10),(100,100),(1000,1000)],
+          nans=[False, False, False, True, True, True]):
+    """
+    Bottleneck benchmark.
+
+    Parameters
+    ----------
+    mode : {'fast', 'faster'}, optional
+        Whether to benchmark the high-level functions ('fast') such as
+        bn.median or the low-level functions ('faster') such as
+        bn.func.median_2d_float64_axis0. By default the high-level functions
+        are benchmarked.
+    dtype : str, optional
+        Data type string such as 'float64', which is the default.
+    axis : int, optional
+        Axis along which to perform the calculations that are being
+        benchmarked. The default is the first axis (axis=0).
+    shapes : list, optional
+        A list of tuple shapes of input arrays to use in the benchmark.
+    nans : list, optional
+        A list of the bools (True or False), one for each tuple in the
+        `shapes` list, that tells whether the input arrays should be filled
+        with one-third NaNs.
+
+    Returns
+    -------
+    A benchmark report is printed to stdout.
+
+    """
+    
+    try:
+        import scipy as sp
+        SCIPY = True
+    except ImportError:
+        SCIPY = False
+
+    if len(shapes) != len(nans):
+        raise ValueError("`shapes` and `nans` must have the same length")
 
     dtype = str(dtype)
     axis = str(axis)
@@ -33,26 +70,32 @@ def bench(mode='fast', dtype='float64', axis=0):
     elif mode == 'faster':    
         print "%sLow-level functions used (mode='faster')" % tab
     
-    h1 = "%s no NaN   no NaN     no NaN     NaN      NaN        NaN"
-    h2 = "%s(10,10) (100,100) (1000,1000) (10,10) (100,100) (1000,1000)"
     print
-    print h1 % (16*" ")
-    print h2 % (16*" ")
+    header = [" "*14]
+    for nan in nans:
+        if nan:
+            header.append("NaN".center(11))
+        else:
+            header.append("no NaN".center(11))
+    print "".join(header)       
+    header = ["".join(str(shape).split(" ")).center(11) for shape in shapes]
+    header = [" "*14] + header
+    print "".join(header)
 
-    suite = benchsuite(mode, dtype, axis)
-    fmt = "%s%6.2f   %6.2f     %6.2f    %6.2f   %6.2f     %6.2f"
+    suite = benchsuite(mode, shapes, dtype, axis, nans)
     for test in suite:
-        name = test["name"].ljust(16)
+        name = test["name"].ljust(12)
+        fmt = name + "%10.2f" + "%11.2f"*(len(shapes) - 1)
         if test['scipy_required'] and not SCIPY:
             print "%s%s" % (name, "requires SciPy")
         else:
-            s = timer(test['statements'], test['setups'])
-            print fmt % (name, s[0], s[1], s[2], s[3], s[4], s[5])
+            speed = timer(test['statements'], test['setups'])
+            print fmt % tuple(speed)
 
     print
     print 'Reference functions:'
     for test in suite:
-        print "%s%s" % (test["name"].ljust(16), test['ref'])
+        print "%s%s" % (test["name"].ljust(15), test['ref'])
 
 def timer(statements, setups):
     speed = []
@@ -74,27 +117,22 @@ def getarray(shape, dtype, nans=False):
         rs.shuffle(arr)
     return arr.reshape(*shape)
     
-def benchsuite(mode, dtype, axis):
+def benchsuite(mode, shapes, dtype, axis, nans):
 
     if mode not in ('fast', 'faster'):
         raise ValueError("`mode` must be 'fast' or 'faster'")
 
     suite = []
    
-    def getsetups(setup):
+    def getsetups(setup, shapes, nans):
         template = """import numpy as np
         import bottleneck as bn
         from bottleneck.benchmark.bench import getarray
-        N = %d
-        a = getarray((N,N), 'DTYPE', %s)
+        a = getarray(%s, 'DTYPE', %s)
         %s"""
         setups = []
-        setups.append(template % (10, str(False), setup))
-        setups.append(template % (100, str(False), setup))
-        setups.append(template % (1000, str(False), setup))
-        setups.append(template % (10, str(True), setup))
-        setups.append(template % (100, str(True), setup))
-        setups.append(template % (1000, str(True), setup))
+        for shape, nan in zip(shapes, nans):
+            setups.append(template % (str(shape), str(nan), setup))
         return setups
 
     # median
@@ -110,7 +148,7 @@ def benchsuite(mode, dtype, axis):
     setup = """
         func, a = bn.func.median_selector(a, axis=AXIS)
     """    
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     suite.append(run)
     
     # nanmedian
@@ -127,7 +165,7 @@ def benchsuite(mode, dtype, axis):
         from bottleneck.slow.func import scipy_nanmedian
         func, a = bn.func.nanmedian_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     suite.append(run)
     
     # nansum
@@ -143,7 +181,7 @@ def benchsuite(mode, dtype, axis):
     setup = """
         func, a = bn.func.nansum_selector(a, axis=AXIS)
     """    
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     suite.append(run)
 
     # nanmax
@@ -159,7 +197,7 @@ def benchsuite(mode, dtype, axis):
     setup = """
         func, a = bn.func.nanmax_selector(a, axis=AXIS)
     """    
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     suite.append(run)
     
     # nanmean
@@ -176,7 +214,7 @@ def benchsuite(mode, dtype, axis):
         from bottleneck.slow.func import scipy_nanmean
         func, a = bn.func.nanmean_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     suite.append(run)
 
     # nanstd
@@ -193,7 +231,7 @@ def benchsuite(mode, dtype, axis):
         from bottleneck.slow.func import scipy_nanstd
         func, a = bn.func.nanstd_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     suite.append(run)
     
     # nanargmax
@@ -209,7 +247,79 @@ def benchsuite(mode, dtype, axis):
     setup = """
         func, a = bn.func.nanargmax_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
+    suite.append(run)
+
+    # ss
+    run = {}
+    run['name'] = "ss"
+    run['ref'] = "scipy.stats.ss"
+    run['scipy_required'] = False
+    if mode == 'fast':
+        code = "bn.ss(a, axis=AXIS)"
+    else:
+        code = "func(a)"
+    run['statements'] = [code, "scipy_ss(a, axis=AXIS)"] 
+    setup = """
+        from bottleneck.slow.func import scipy_ss
+        func, a = bn.func.ss_selector(a, axis=AXIS)
+    """    
+    run['setups'] = getsetups(setup, shapes, nans)
+    suite.append(run)
+    
+    # rankdata
+    run = {}
+    run['name'] = "rankdata"
+    run['ref'] = "scipy.stats.rankdata based (axis support added)"
+    run['scipy_required'] = False
+    if mode == 'fast':
+        code = "bn.rankdata(a, axis=AXIS)"
+    else:
+        code = "func(a)"
+    run['statements'] = [code, "bn.slow.rankdata(a, axis=AXIS)"] 
+    setup = """
+        ignore = bn.slow.rankdata(a, axis=AXIS)
+        func, a = bn.func.rankdata_selector(a, axis=AXIS)
+    """
+    run['setups'] = getsetups(setup, shapes, nans)
+    suite.append(run)
+    
+    # partsort
+    run = {}
+    run['name'] = "partsort"
+    run['ref'] = "np.sort, n=max(a.shape[%s]/2,1)" % axis
+    run['scipy_required'] = False
+    if mode == 'fast':
+        code = "bn.partsort(a, n=n, axis=AXIS)"
+    else:
+        code = "func(a, n)"
+    run['statements'] = [code, "np.sort(a, axis=AXIS)"] 
+    setup = """
+        if AXIS is None: n = a.size
+        else: n = a.shape[AXIS]
+        n = max(n / 2, 1)
+        func, a = bn.func.partsort_selector(a, axis=AXIS)
+    """
+    run['setups'] = getsetups(setup, shapes, nans)
+    suite.append(run)
+    
+    # argpartsort
+    run = {}
+    run['name'] = "argpartsort"
+    run['ref'] = "np.argsort, n=max(a.shape[%s]/2,1)" % axis
+    run['scipy_required'] = False
+    if mode == 'fast':
+        code = "bn.argpartsort(a, n=n, axis=AXIS)"
+    else:
+        code = "func(a, n)"
+    run['statements'] = [code, "np.argsort(a, axis=AXIS)"] 
+    setup = """
+        if AXIS is None: n = a.size
+        else: n = a.shape[AXIS]
+        n = max(n / 2, 1)
+        func, a = bn.func.argpartsort_selector(a, axis=AXIS)
+    """
+    run['setups'] = getsetups(setup, shapes, nans)
     suite.append(run)
     
     # move_sum
@@ -226,9 +336,10 @@ def benchsuite(mode, dtype, axis):
     setup = """
         from bottleneck.slow.move import move_sum as scipy_move_sum
         w = a.shape[AXIS] / 5
+        ignore = bn.slow.move_sum(a, window=w, axis=AXIS)
         func, a = bn.move.move_sum_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     if axis != 'None':
         suite.append(run)
     
@@ -246,9 +357,10 @@ def benchsuite(mode, dtype, axis):
     setup = """
         from bottleneck.slow.move import move_nansum as scipy_move_nansum
         w = a.shape[AXIS] / 5
+        ignore = bn.slow.move_nansum(a, window=w, axis=AXIS)
         func, a = bn.move.move_nansum_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     if axis != 'None':
         suite.append(run)
     
@@ -266,9 +378,10 @@ def benchsuite(mode, dtype, axis):
     setup = """
         from bottleneck.slow.move import move_mean as scipy_move_mean
         w = a.shape[AXIS] / 5
+        ignore = bn.slow.move_mean(a, window=w, axis=AXIS)
         func, a = bn.move.move_mean_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     if axis != 'None':
         suite.append(run)
     
@@ -286,9 +399,10 @@ def benchsuite(mode, dtype, axis):
     setup = """
         from bottleneck.slow.move import move_nanmean as scipy_move_nanmean
         w = a.shape[AXIS] / 5
+        ignore = bn.slow.move_nanmean(a, window=w, axis=AXIS)
         func, a = bn.move.move_nanmean_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     if axis != 'None':
         suite.append(run)
     
@@ -306,9 +420,10 @@ def benchsuite(mode, dtype, axis):
     setup = """
         from bottleneck.slow.move import move_std as scipy_move_std
         w = a.shape[AXIS] / 5
+        ignore = bn.slow.move_std(a, window=w, axis=AXIS)
         func, a = bn.move.move_std_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     if axis != 'None':
         suite.append(run)
     
@@ -326,9 +441,10 @@ def benchsuite(mode, dtype, axis):
     setup = """
         from bottleneck.slow.move import move_nanstd as scipy_move_nanstd
         w = a.shape[AXIS] / 5
+        ignore = bn.slow.move_nanstd(a, window=w, axis=AXIS)
         func, a = bn.move.move_nanstd_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     if axis != 'None':
         suite.append(run)
     
@@ -346,9 +462,10 @@ def benchsuite(mode, dtype, axis):
     setup = """
         from bottleneck.slow.move import move_max as scipy_move_max
         w = a.shape[AXIS] / 5
+        ignore = bn.slow.move_max(a, window=w, axis=AXIS)
         func, a = bn.move.move_max_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     if axis != 'None':
         suite.append(run)
     
@@ -366,9 +483,10 @@ def benchsuite(mode, dtype, axis):
     setup = """
         from bottleneck.slow.move import move_nanmax as scipy_move_nanmax
         w = a.shape[AXIS] / 5
+        ignore = bn.slow.move_nanmax(a, window=w, axis=AXIS)
         func, a = bn.move.move_nanmax_selector(a, axis=AXIS)
     """
-    run['setups'] = getsetups(setup)
+    run['setups'] = getsetups(setup, shapes, nans)
     if axis != 'None':
         suite.append(run)
     
