@@ -2,7 +2,8 @@
 import numpy as np
 
 __all__ = ['median', 'nanmedian', 'nansum', 'nanmean', 'nanvar', 'nanstd',
-           'nanmin', 'nanmax', 'nanargmin', 'nanargmax']
+           'nanmin', 'nanmax', 'nanargmin', 'nanargmax', 'rankdata',
+           'nanrankdata', 'ss', 'partsort', 'argpartsort']
 
 def median(arr, axis=None):
     "Slow median function used for unaccelerated ndim/dtype combinations."
@@ -82,11 +83,19 @@ def nanstd(arr, axis=None, ddof=0):
 
 def nanmin(arr, axis=None):
     "Slow nanmin function used for unaccelerated ndim/dtype combinations."
-    return np.nanmin(arr, axis=axis)
+    y =  np.nanmin(arr, axis=axis)
+    if not hasattr(y, "dtype"):
+        # Numpy 1.5.1 doesn't return object with dtype when input is all NaN
+        y = arr.dtype.type(y)
+    return y
 
 def nanmax(arr, axis=None):
     "Slow nanmax function used for unaccelerated ndim/dtype combinations."
-    return np.nanmax(arr, axis=axis)
+    y =  np.nanmax(arr, axis=axis)
+    if not hasattr(y, "dtype"):
+        # Numpy 1.5.1 doesn't return object with dtype when input is all NaN
+        y = arr.dtype.type(y)
+    return y
 
 def nanargmin(arr, axis=None):
     "Slow nanargmin function used for unaccelerated ndim/dtype combinations."
@@ -96,6 +105,52 @@ def nanargmax(arr, axis=None):
     "Slow nanargmax function used for unaccelerated ndim/dtype combinations."
     return np.nanargmax(arr, axis=axis)
 
+def rankdata(arr, axis=None):
+    "Slow rankdata function used for unaccelerated ndim/dtype combinations."
+    if axis is None:
+        arr = arr.ravel()
+        axis = 0
+    elif axis < 0:
+        axis = range(arr.ndim)[axis]
+    y = np.empty(arr.shape)
+    itshape = list(arr.shape)
+    itshape.pop(axis)
+    for ij in np.ndindex(*itshape):
+        ijslice = list(ij[:axis]) + [slice(None)] + list(ij[axis:])
+        y[ijslice] = scipy_rankdata(arr[ijslice].astype('float'))
+    return y
+
+def nanrankdata(arr, axis=None):
+    "Slow nanrankdata function used for unaccelerated ndim/dtype combinations."
+    if axis is None:
+        arr = arr.ravel()
+        axis = 0
+    elif axis < 0:
+        axis = range(arr.ndim)[axis]
+    y = np.empty(arr.shape)
+    y.fill(np.nan)
+    itshape = list(arr.shape)
+    itshape.pop(axis)
+    for ij in np.ndindex(*itshape):
+        ijslice = list(ij[:axis]) + [slice(None)] + list(ij[axis:])
+        x1d = arr[ijslice].astype(float)
+        mask1d = ~np.isnan(x1d)
+        x1d[mask1d] = scipy_rankdata(x1d[mask1d])
+        y[ijslice] = x1d
+    return y
+
+def ss(arr, axis=0):
+    "Slow sum of squares used for unaccelerated ndim/dtype combinations."
+    return scipy_ss(arr, axis)
+
+def partsort(arr, n, axis=-1):
+    "Slow partial sort used for unaccelerated ndim/dtype combinations."
+    return np.sort(arr, axis)
+
+def argpartsort(arr, n, axis=-1):
+    "Slow partial argsort used for unaccelerated ndim/dtype combinations."
+    return np.argsort(arr, axis)
+
 # ---------------------------------------------------------------------------
 #
 # SciPy
@@ -105,7 +160,8 @@ def nanargmax(arr, axis=None):
 # which is distributed with Bottleneck.
 #
 # Code taken from scipy trunk on Dec 16, 2010.
-# nanmedian take from scipy trunk on Dec 17, 2010.
+# nanmedian taken from scipy trunk on Dec 17, 2010.
+# rankdata taken from scipy HEAD on Mar 16, 2011.
 
 def scipy_nanmean(x, axis=0):
     """
@@ -227,6 +283,7 @@ def _nanmedian(arr1d):  # This only works on 1d arrays
         return np.nan
     return np.median(x)
 
+# Feb 2011: patched nanmedian to handle nanmedian(a, 1) with a = np.ones((2,0))
 def scipy_nanmedian(x, axis=0):
     """
     Compute the median along the given axis ignoring nan values.
@@ -277,10 +334,15 @@ def scipy_nanmedian(x, axis=0):
     x, axis = _chk_asarray(x, axis)
     if x.ndim == 0:
         return float(x.item())
-    x = x.copy()
-    x = np.apply_along_axis(_nanmedian, axis, x)
-    if x.ndim == 0:
-        x = float(x.item())
+    shape = list(x.shape)
+    shape.pop(axis)
+    if 0 in shape:
+        x = np.empty(shape)
+    else:    
+        x = x.copy()
+        x = np.apply_along_axis(_nanmedian, axis, x)
+        if x.ndim == 0:
+            x = float(x.item())
     return x
 
 def _chk_asarray(a, axis):
@@ -291,3 +353,102 @@ def _chk_asarray(a, axis):
         a = np.asarray(a)
         outaxis = axis
     return a, outaxis
+
+def fastsort(a):
+    """
+    Sort an array and provide the argsort.
+
+    Parameters
+    ----------
+    a : array_like
+        Input array.
+
+    Returns
+    -------
+    fastsort : ndarray of type int
+        sorted indices into the original array
+
+    """
+    # TODO: the wording in the docstring is nonsense.
+    it = np.argsort(a)
+    as_ = a[it]
+    return as_, it
+
+def scipy_rankdata(a):
+    """
+    Ranks the data, dealing with ties appropriately.
+
+    Equal values are assigned a rank that is the average of the ranks that
+    would have been otherwise assigned to all of the values within that set.
+    Ranks begin at 1, not 0.
+
+    Parameters
+    ----------
+    a : array_like
+        This array is first flattened.
+
+    Returns
+    -------
+    rankdata : ndarray
+         An array of length equal to the size of `a`, containing rank scores.
+
+    Examples
+    --------
+    >>> stats.rankdata([0, 2, 2, 3])
+    array([ 1. ,  2.5,  2.5,  4. ])
+
+    """
+    a = np.ravel(a)
+    n = len(a)
+    svec, ivec = fastsort(a)
+    sumranks = 0
+    dupcount = 0
+    newarray = np.zeros(n, float)
+    for i in xrange(n):
+        sumranks += i
+        dupcount += 1
+        if i==n-1 or svec[i] != svec[i+1]:
+            averank = sumranks / float(dupcount) + 1
+            for j in xrange(i-dupcount+1,i+1):
+                newarray[ivec[j]] = averank
+            sumranks = 0
+            dupcount = 0
+    return newarray
+
+def scipy_ss(a, axis=0):
+    """
+    Squares each element of the input array, and returns the square(s) of that.
+
+    Parameters
+    ----------
+    a : array_like
+        Input array.
+    axis : int or None, optional
+        The axis along which to calculate. If None, use whole array.
+        Default is 0, i.e. along the first axis.
+
+    Returns
+    -------
+    ss : ndarray
+        The sum along the given axis for (a**2).
+
+    See also
+    --------
+    square_of_sums : The square(s) of the sum(s) (the opposite of `ss`).
+
+    Examples
+    --------
+    >>> from scipy import stats
+    >>> a = np.array([1., 2., 5.])
+    >>> stats.ss(a)
+    30.0
+
+    And calculating along an axis:
+
+    >>> b = np.array([[1., 2., 5.], [2., 5., 6.]])
+    >>> stats.ss(b, axis=1)
+    array([ 30., 65.])
+
+    """
+    a, axis = _chk_asarray(a, axis)
+    return np.sum(a*a, axis)
