@@ -12,21 +12,23 @@ INT_DTYPES = [x for x in bn.dtypes if 'int' in x]
 
 loop = {}
 loop[1] = """\
+    if nINDEX0 == 0:
+        return np.FLOAT(NAN)
     k = nAXIS >> 1
     l = 0
     r = nAXIS - 1 
     with nogil:       
         while l < r:
-            x = a[k]
+            x = b[k]
             i = l
             j = r
             while 1:
-                while a[i] < x: i += 1
-                while x < a[j]: j -= 1
+                while b[i] < x: i += 1
+                while x < b[j]: j -= 1
                 if i <= j:
-                    tmp = a[i]
-                    a[i] = a[j]
-                    a[j] = tmp
+                    tmp = b[i]
+                    b[i] = b[j]
+                    b[j] = tmp
                     i += 1
                     j -= 1
                 if i > j: break
@@ -35,29 +37,32 @@ loop[1] = """\
     if nAXIS % 2 == 0:        
         amax = MINDTYPE
         for i in range(k):
-            ai = a[i]
+            ai = b[i]
             if ai >= amax:
                 amax = ai
-        return np.FLOAT(0.5 * (a[k] + amax))
+        return np.FLOAT(0.5 * (b[k] + amax))
     else:
-        return np.FLOAT(a[k])
+        return np.FLOAT(b[k])
 """        
 loop[2] = """\
+    if nINDEX1 == 0:
+        PyArray_FillWithScalar(y, NAN)
+        return y
     for iINDEX0 in range(nINDEX0): 
         k = nAXIS >> 1
         l = 0
         r = nAXIS - 1
         while l < r:
-            x = a[INDEXREPLACE|k|]
+            x = b[INDEXREPLACE|k|]
             i = l
             j = r
             while 1:
-                while a[INDEXREPLACE|i|] < x: i += 1
-                while x < a[INDEXREPLACE|j|]: j -= 1
+                while b[INDEXREPLACE|i|] < x: i += 1
+                while x < b[INDEXREPLACE|j|]: j -= 1
                 if i <= j:
-                    tmp = a[INDEXREPLACE|i|]
-                    a[INDEXREPLACE|i|] = a[INDEXREPLACE|j|]
-                    a[INDEXREPLACE|j|] = tmp
+                    tmp = b[INDEXREPLACE|i|]
+                    b[INDEXREPLACE|i|] = b[INDEXREPLACE|j|]
+                    b[INDEXREPLACE|j|] = tmp
                     i += 1
                     j -= 1
                 if i > j: break
@@ -66,31 +71,34 @@ loop[2] = """\
         if nAXIS % 2 == 0:        
             amax = MINDTYPE
             for i in range(k):
-                ai = a[INDEXREPLACE|i|]
+                ai = b[INDEXREPLACE|i|]
                 if ai >= amax:
                     amax = ai
-            y[INDEXPOP] = 0.5 * (a[INDEXREPLACE|k|] + amax)
+            y[INDEXPOP] = 0.5 * (b[INDEXREPLACE|k|] + amax)
         else:
-            y[INDEXPOP] = CASTa[INDEXREPLACE|k|]         
+            y[INDEXPOP] = CASTb[INDEXREPLACE|k|]         
     return y
 """
 loop[3] = """\
+    if nINDEX2 == 0:
+        PyArray_FillWithScalar(y, NAN)
+        return y
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
             k = nAXIS >> 1
             l = 0
             r = nAXIS - 1
             while l < r:
-                x = a[INDEXREPLACE|k|]
+                x = b[INDEXREPLACE|k|]
                 i = l
                 j = r
                 while 1:
-                    while a[INDEXREPLACE|i|] < x: i += 1
-                    while x < a[INDEXREPLACE|j|]: j -= 1
+                    while b[INDEXREPLACE|i|] < x: i += 1
+                    while x < b[INDEXREPLACE|j|]: j -= 1
                     if i <= j:
-                        tmp = a[INDEXREPLACE|i|]
-                        a[INDEXREPLACE|i|] = a[INDEXREPLACE|j|]
-                        a[INDEXREPLACE|j|] = tmp
+                        tmp = b[INDEXREPLACE|i|]
+                        b[INDEXREPLACE|i|] = b[INDEXREPLACE|j|]
+                        b[INDEXREPLACE|j|] = tmp
                         i += 1
                         j -= 1
                     if i > j: break
@@ -99,12 +107,12 @@ loop[3] = """\
             if nAXIS % 2 == 0:        
                 amax = MINDTYPE
                 for i in range(k):
-                    ai = a[INDEXREPLACE|i|]
+                    ai = b[INDEXREPLACE|i|]
                     if ai >= amax:
                         amax = ai
-                y[INDEXPOP] = 0.5 * (a[INDEXREPLACE|k|] + amax)
+                y[INDEXPOP] = 0.5 * (b[INDEXREPLACE|k|] + amax)
             else:
-                y[INDEXPOP] = CASTa[INDEXREPLACE|k|]         
+                y[INDEXPOP] = CASTb[INDEXREPLACE|k|]         
     return y
 """
 
@@ -123,6 +131,7 @@ def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
     "Median of NDIMd array with dtype=DTYPE along axis=AXIS."
     cdef np.npy_intp i, j = 0, l, r, k 
     cdef np.DTYPE_t x, tmp, amax, ai
+    cdef np.ndarray[np.DTYPE_t, ndim=NDIM] b = PyArray_Copy(a)
 """
 
 floats['loop'] = {}
@@ -157,7 +166,7 @@ median['slow'] = slow
 median['templates'] = {}
 median['templates']['float'] = floats
 median['templates']['int'] = ints
-median['pyx_file'] = 'func/median.pyx'
+median['pyx_file'] = 'func/%sbit/median.pyx'
 
 median['main'] = '''"median auto-generated from template"
 # Select smallest k elements code used for inner loop of median method:
@@ -239,9 +248,8 @@ def median_selector(arr, axis):
     ----------
     arr : array_like
         Input array. If `arr` is not an array, a conversion is attempted.
-    axis : {int, None}, optional
-        Axis along which the median is to be computed. The default (axis=None)
-        is to compute the mean of the flattened array.
+    axis : {int, None}
+        Axis along which the median is to be computed.
     
     Returns
     -------
@@ -283,7 +291,7 @@ def median_selector(arr, axis):
         if axis < 0:
             axis += ndim
     else:
-        a = a.ravel()
+        a = PyArray_Ravel(a, NPY_CORDER)
         axis = 0
         ndim = 1
     key = (ndim, dtype, axis)
