@@ -13,7 +13,7 @@ INT_DTYPES = [x for x in bn.dtypes if 'int' in x]
 floats = {}
 floats['dtypes'] = FLOAT_DTYPES
 floats['axisNone'] = False
-floats['force_output_dtype'] = 'NPINT'
+floats['force_output_dtype'] = 'intp'
 floats['reuse_non_nan_func'] = False
 
 floats['top'] = """
@@ -28,6 +28,9 @@ def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
 
 loop = {}
 loop[1] = """\
+    if nINDEX0 == 0:
+        msg = "numpy.nanargmin raises on a.shape[axis]==0; Bottleneck too."
+        raise ValueError(msg)
     amin = MAXDTYPE
     for iINDEX0 in range(nINDEX0 - 1, -1, -1):
         ai = a[INDEXALL]
@@ -36,11 +39,14 @@ loop[1] = """\
             allnan = 0
             idx = iINDEX0
     if allnan == 0:       
-        return np.NPINT(idx)
+        return np.intp(idx)
     else:
         return NAN
 """
 loop[2] = """\
+    if nINDEX1 == 0:
+        msg = "numpy.nanargmin raises on a.shape[axis]==0; Bottleneck too."
+        raise ValueError(msg)
     for iINDEX0 in range(nINDEX0 - 1, -1, -1):
         amin = MAXDTYPE
         allnan = 1
@@ -57,6 +63,9 @@ loop[2] = """\
     return y
 """
 loop[3] = """\
+    if nINDEX2 == 0:
+        msg = "numpy.nanargmin raises on a.shape[axis]==0; Bottleneck too." 
+        raise ValueError(msg)
     for iINDEX0 in range(nINDEX0 - 1, -1, -1):
         for iINDEX1 in range(nINDEX1 - 1, -1, -1):
             amin = MAXDTYPE
@@ -75,7 +84,6 @@ loop[3] = """\
 """
 
 floats['loop'] = loop
-#floats['loop'][1] = floats['loop'][1].replace('DTYPE2', NPINT)
 
 # Int dtypes (not axis=None) ------------------------------------------------
 
@@ -84,15 +92,21 @@ ints['dtypes'] = INT_DTYPES
 
 loop = {}
 loop[1] = """\
+    if nINDEX0 == 0:
+        msg = "numpy.nanargmin raises on a.shape[axis]==0; Bottleneck too."
+        raise ValueError(msg)
     amin = MAXDTYPE
     for iINDEX0 in range(nINDEX0):
         ai = a[INDEXALL]
         if ai <= amin:
             amin = ai
             idx = iINDEX0
-    return np.NPINT(idx)
+    return np.intp(idx)
 """
 loop[2] = """\
+    if nINDEX1 == 0:
+        msg = "numpy.nanargmin raises on a.shape[axis]==0; Bottleneck too."
+        raise ValueError(msg)
     for iINDEX0 in range(nINDEX0 - 1, -1, -1):
         amin = MAXDTYPE
         for iINDEX1 in range(nINDEX1 - 1, -1, -1):
@@ -104,6 +118,9 @@ loop[2] = """\
     return y
 """
 loop[3] = """\
+    if nINDEX2 == 0:
+        msg = "numpy.nanmin raises on a.shape[axis]==0; so Bottleneck does."
+        raise ValueError(msg)
     for iINDEX0 in range(nINDEX0 - 1, -1, -1):
         for iINDEX1 in range(nINDEX1 - 1, -1, -1):
             amin = MAXDTYPE
@@ -117,7 +134,6 @@ loop[3] = """\
 """
 
 ints['loop'] = loop
-#ints['loop'][1] = ints['loop'][1].replace('DTYPE2', NPINT)
 
 # Slow, unaccelerated ndim/dtype --------------------------------------------
 
@@ -136,7 +152,7 @@ nanargmin['slow'] = slow
 nanargmin['templates'] = {}
 nanargmin['templates']['float'] = floats
 nanargmin['templates']['int'] = ints
-nanargmin['pyx_file'] = 'func/nanargmin.pyx'
+nanargmin['pyx_file'] = 'func/%sbit/nanargmin.pyx'
 
 nanargmin['main'] = '''"nanargmin auto-generated from template"
 
@@ -198,9 +214,8 @@ def nanargmin_selector(arr, axis):
     ----------
     arr : array_like
         Input array. If `arr` is not an array, a conversion is attempted.
-    axis : {int, None}, optional
-        Axis along which the indices are found. The default (axis=None) is to
-        find the index of the minimum value in the flattened array.
+    axis : {int, None}
+        Axis along which the indices are found.
     
     Returns
     -------
@@ -237,15 +252,11 @@ def nanargmin_selector(arr, axis):
         a = np.array(arr, copy=False)
     cdef int ndim = PyArray_NDIM(a)
     cdef int dtype = PyArray_TYPE(a)
-    cdef int size = PyArray_SIZE(a)
-    if size == 0:
-        msg = "numpy.nanargmin() raises on size=0; so Bottleneck does too." 
-        raise ValueError, msg
     if axis is not None:
         if axis < 0:
             axis += ndim
     else:
-        a = a.ravel()
+        a = PyArray_Ravel(a, NPY_CORDER)
         axis = 0
         ndim = 1
     cdef tuple key = (ndim, dtype, axis)
