@@ -20,7 +20,7 @@ floats['top'] = """
 @cython.boundscheck(False)
 @cython.wraparound(False)
 def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
-    "Mean of NDIMd array with dtype=DTYPE along axis=0 ignoring NaNs."
+    "Mean of NDIMd array with dtype=DTYPE along axis=AXIS ignoring NaNs."
     cdef int count = 0
     cdef np.DTYPE_t asum = 0, ai
 """
@@ -108,26 +108,32 @@ ints['top'] = """
 @cython.boundscheck(False)
 @cython.wraparound(False)
 def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
-    "Mean of NDIMd array with dtype=DTYPE along axis=0 ignoring NaNs."
+    "Mean of NDIMd array with dtype=DTYPE along axis=AXIS ignoring NaNs."
     cdef np.float64_t asum = 0, ai
 """
 
 loop = {}
 loop[2] = """\
-    for iINDEX0 in range(nINDEX0):
-        asum = 0
-        for iINDEX1 in range(nINDEX1):
-            asum += a[INDEXALL]
-        y[INDEXPOP] = asum / nINDEX1
+    if nINDEX1 == 0:
+        PyArray_FillWithScalar(y, NAN)
+    else:
+        for iINDEX0 in range(nINDEX0):
+            asum = 0
+            for iINDEX1 in range(nINDEX1):
+                asum += a[INDEXALL]
+            y[INDEXPOP] = asum / nINDEX1
     return y
 """
 loop[3] = """\
-    for iINDEX0 in range(nINDEX0):
-        for iINDEX1 in range(nINDEX1):
-            asum = 0
-            for iINDEX2 in range(nINDEX2):
-                asum += a[INDEXALL]
-            y[INDEXPOP] = asum / nINDEX2
+    if nINDEX2 == 0:
+        PyArray_FillWithScalar(y, NAN)
+    else:
+        for iINDEX0 in range(nINDEX0):
+            for iINDEX1 in range(nINDEX1):
+                asum = 0
+                for iINDEX2 in range(nINDEX2):
+                    asum += a[INDEXALL]
+                y[INDEXPOP] = asum / nINDEX2
     return y
 """
 ints['loop'] = loop
@@ -143,14 +149,20 @@ loop[1] = """\
     size = nINDEX0
     for iINDEX0 in range(nINDEX0):
         asum += a[INDEXALL]
-    return np.float64(asum / size)
+    if size > 0:    
+        return np.float64(asum / size)
+    else:
+        return np.float64(NAN)
 """
 loop[2] = """\
     size = nINDEX0 * nINDEX1    
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
             asum += a[INDEXALL]
-    return np.float64(asum / size)
+    if size > 0:    
+        return np.float64(asum / size)
+    else:
+        return np.float64(NAN)
 """
 loop[3] = """\
     size = nINDEX0 * nINDEX1 * nINDEX2 
@@ -158,7 +170,10 @@ loop[3] = """\
         for iINDEX1 in range(nINDEX1):
             for iINDEX2 in range(nINDEX2):
                 asum += a[INDEXALL]
-    return np.float64(asum / size)
+    if size > 0:    
+        return np.float64(asum / size)
+    else:
+        return np.float64(NAN)
 """
 ints_None['loop'] = loop
 
@@ -181,7 +196,7 @@ nanmean['templates']['float'] = floats
 nanmean['templates']['float_None'] = floats_None
 nanmean['templates']['int'] = ints
 nanmean['templates']['int_None'] = ints_None
-nanmean['pyx_file'] = 'func/nanmean.pyx'
+nanmean['pyx_file'] = 'func/%sbit/nanmean.pyx'
 
 nanmean['main'] = '''"nanmean auto-generated from template"
 
@@ -264,9 +279,8 @@ def nanmean_selector(arr, axis):
     ----------
     arr : array_like
         Input array. If `arr` is not an array, a conversion is attempted.
-    axis : {int, None}, optional
-        Axis along which the mean is to be computed. The default (axis=None)
-        is to compute the mean of the flattened array.
+    axis : {int, None}
+        Axis along which the mean is to be computed.
     
     Returns
     -------
