@@ -12,43 +12,38 @@ INT_DTYPES = [x for x in bn.dtypes if 'int' in x]
 
 loop = {}
 loop[1] = """\
-    k = nAXIS 
-    l = 0
-    r = k - 1
-    while l < r:
-        i = l
-        j = r
-        while a[i] == a[i]:
-            i += 1
-            if i == nAXIS:
+    if nINDEX0 == 0:
+        return np.FLOAT(NAN)
+    j = nAXIS - 1
+    flag = 1
+    for i in range(nINDEX0):
+        if b[i] != b[i]:
+            while b[j] != b[j]:
+                if j <= 0:
+                    break
+                j -= 1    
+            if i >= j:
+                flag = 0
                 break
-        while a[j] != a[j]:
-            j -= 1
-        if i <= j:
-            tmp = a[i]
-            a[i] = a[j]
-            a[j] = tmp
-            i += 1
-            j -= 1
-        if i > j: break
-        l = i
-        r = j
-    n = j + 1 
+            tmp = b[i]
+            b[i] = b[j]
+            b[j] = tmp
+    n = i + flag
     k = n >> 1
     l = 0
     r = n - 1 
     with nogil:       
         while l < r:
-            x = a[k]
+            x = b[k]
             i = l
             j = r
             while 1:
-                while a[i] < x: i += 1
-                while x < a[j]: j -= 1
+                while b[i] < x: i += 1
+                while x < b[j]: j -= 1
                 if i <= j:
-                    tmp = a[i]
-                    a[i] = a[j]
-                    a[j] = tmp
+                    tmp = b[i]
+                    b[i] = b[j]
+                    b[j] = tmp
                     i += 1
                     j -= 1
                 if i > j: break
@@ -56,52 +51,53 @@ loop[1] = """\
             if k < i: r = j
     if n % 2 == 0:        
         amax = MINDTYPE
+        allnan = 1
         for i in range(k):
-            ai = a[i]
+            ai = b[i]
             if ai >= amax:
                 amax = ai
-        return np.FLOAT(0.5 * (a[k] + amax))
+                allnan = 0
+        if allnan == 0:        
+            return np.FLOAT(0.5 * (b[k] + amax))
+        else:
+            return np.FLOAT(b[k])
     else:
-        return np.FLOAT(a[k])
+        return np.FLOAT(b[k])
 """        
 loop[2] = """\
-    for iINDEX0 in range(nINDEX0): 
-        k = nAXIS 
-        l = 0
-        r = k - 1
-        while l < r:
-            i = l
-            j = r
-            while a[INDEXREPLACE|i|] == a[INDEXREPLACE|i|]:
-                i += 1
-                if i == nAXIS:
+    if nINDEX1 == 0:
+        PyArray_FillWithScalar(y, NAN)
+        return y
+    for iINDEX0 in range(nINDEX0):
+        j = nAXIS - 1
+        flag = 1
+        for i in range(nINDEX1):
+            if b[INDEXREPLACE|i|] != b[INDEXREPLACE|i|]:
+                while b[INDEXREPLACE|j|] != b[INDEXREPLACE|j|]:
+                    if j <= 0:
+                        break
+                    j -= 1    
+                if i >= j:
+                    flag = 0
                     break
-            while a[INDEXREPLACE|j|] != a[INDEXREPLACE|j|]:
-                j -= 1
-            if i <= j:
-                tmp = a[INDEXREPLACE|i|]
-                a[INDEXREPLACE|i|] = a[INDEXREPLACE|j|]
-                a[INDEXREPLACE|j|] = tmp
-                i += 1
-                j -= 1
-            if i > j: break
-            l = i
-            r = j
-        n = j + 1 
+                tmp = b[INDEXREPLACE|i|]
+                b[INDEXREPLACE|i|] = b[INDEXREPLACE|j|]
+                b[INDEXREPLACE|j|] = tmp
+        n = i + flag
         k = n >> 1
         l = 0
         r = n - 1
         while l < r:
-            x = a[INDEXREPLACE|k|]
+            x = b[INDEXREPLACE|k|]
             i = l
             j = r
             while 1:
-                while a[INDEXREPLACE|i|] < x: i += 1
-                while x < a[INDEXREPLACE|j|]: j -= 1
+                while b[INDEXREPLACE|i|] < x: i += 1
+                while x < b[INDEXREPLACE|j|]: j -= 1
                 if i <= j:
-                    tmp = a[INDEXREPLACE|i|]
-                    a[INDEXREPLACE|i|] = a[INDEXREPLACE|j|]
-                    a[INDEXREPLACE|j|] = tmp
+                    tmp = b[INDEXREPLACE|i|]
+                    b[INDEXREPLACE|i|] = b[INDEXREPLACE|j|]
+                    b[INDEXREPLACE|j|] = tmp
                     i += 1
                     j -= 1
                 if i > j: break
@@ -109,54 +105,55 @@ loop[2] = """\
             if k < i: r = j
         if n % 2 == 0:        
             amax = MINDTYPE
+            allnan = 1
             for i in range(k):
-                ai = a[INDEXREPLACE|i|]
+                ai = b[INDEXREPLACE|i|]
                 if ai >= amax:
                     amax = ai
-            y[INDEXPOP] = 0.5 * (a[INDEXREPLACE|k|] + amax)
+                    allnan = 0
+            if allnan == 0:        
+                y[INDEXPOP] = 0.5 * (b[INDEXREPLACE|k|] + amax)
+            else:
+                y[INDEXPOP] = CASTb[INDEXREPLACE|k|]         
         else:
-            y[INDEXPOP] = CASTa[INDEXREPLACE|k|]         
+            y[INDEXPOP] = CASTb[INDEXREPLACE|k|]         
     return y
 """
 loop[3] = """\
+    if nINDEX2 == 0:
+        PyArray_FillWithScalar(y, NAN)
+        return y
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
-            k = nAXIS 
-            l = 0
-            r = k - 1
-            while l < r:
-                i = l
-                j = r
-                while a[INDEXREPLACE|i|] == a[INDEXREPLACE|i|]:
-                    i += 1
-                    if i == nAXIS:
+            j = nAXIS - 1
+            flag = 1
+            for i in range(nINDEX2):
+                if b[INDEXREPLACE|i|] != b[INDEXREPLACE|i|]:
+                    while b[INDEXREPLACE|j|] != b[INDEXREPLACE|j|]:
+                        if j <= 0:
+                            break
+                        j -= 1    
+                    if i >= j:
+                        flag = 0
                         break
-                while a[INDEXREPLACE|j|] != a[INDEXREPLACE|j|]:
-                    j -= 1
-                if i <= j:
-                    tmp = a[INDEXREPLACE|i|]
-                    a[INDEXREPLACE|i|] = a[INDEXREPLACE|j|]
-                    a[INDEXREPLACE|j|] = tmp
-                    i += 1
-                    j -= 1
-                if i > j: break
-                l = i
-                r = j
-            n = j + 1 
+                    tmp = b[INDEXREPLACE|i|]
+                    b[INDEXREPLACE|i|] = b[INDEXREPLACE|j|]
+                    b[INDEXREPLACE|j|] = tmp
+            n = i + flag
             k = n >> 1
             l = 0
             r = n - 1
             while l < r:
-                x = a[INDEXREPLACE|k|]
+                x = b[INDEXREPLACE|k|]
                 i = l
                 j = r
                 while 1:
-                    while a[INDEXREPLACE|i|] < x: i += 1
-                    while x < a[INDEXREPLACE|j|]: j -= 1
+                    while b[INDEXREPLACE|i|] < x: i += 1
+                    while x < b[INDEXREPLACE|j|]: j -= 1
                     if i <= j:
-                        tmp = a[INDEXREPLACE|i|]
-                        a[INDEXREPLACE|i|] = a[INDEXREPLACE|j|]
-                        a[INDEXREPLACE|j|] = tmp
+                        tmp = b[INDEXREPLACE|i|]
+                        b[INDEXREPLACE|i|] = b[INDEXREPLACE|j|]
+                        b[INDEXREPLACE|j|] = tmp
                         i += 1
                         j -= 1
                     if i > j: break
@@ -164,13 +161,18 @@ loop[3] = """\
                 if k < i: r = j
             if n % 2 == 0:        
                 amax = MINDTYPE
+                allnan = 1
                 for i in range(k):
-                    ai = a[INDEXREPLACE|i|]
+                    ai = b[INDEXREPLACE|i|]
                     if ai >= amax:
                         amax = ai
-                y[INDEXPOP] = 0.5 * (a[INDEXREPLACE|k|] + amax)
+                        allnan = 0
+                if allnan == 0:   
+                    y[INDEXPOP] = 0.5 * (b[INDEXREPLACE|k|] + amax)
+                else:
+                    y[INDEXPOP] = CASTb[INDEXREPLACE|k|]
             else:
-                y[INDEXPOP] = CASTa[INDEXREPLACE|k|]         
+                y[INDEXPOP] = CASTb[INDEXREPLACE|k|]         
     return y
 """
 
@@ -187,8 +189,10 @@ floats['top'] = """
 @cython.wraparound(False)
 def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
     "Median of NDIMd array with dtype=DTYPE along axis=AXIS."
-    cdef np.npy_intp i, j = 0, l, r, k, n 
+    cdef int allnan = 1, flag = 0
+    cdef np.npy_intp i = 0, j = 0, l, r, k, n 
     cdef np.DTYPE_t x, tmp, amax, ai
+    cdef np.ndarray[np.DTYPE_t, ndim=NDIM] b = PyArray_Copy(a)
 """
 
 floats['loop'] = {}
@@ -219,7 +223,7 @@ nanmedian['slow'] = slow
 nanmedian['templates'] = {}
 nanmedian['templates']['float'] = floats
 nanmedian['templates']['int'] = ints
-nanmedian['pyx_file'] = 'func/nanmedian.pyx'
+nanmedian['pyx_file'] = 'func/%sbit/nanmedian.pyx'
 
 nanmedian['main'] = '''"nanmedian auto-generated from template"
 # Select smallest k elements code used for inner loop of median method:
@@ -296,9 +300,8 @@ def nanmedian_selector(arr, axis):
     ----------
     arr : array_like
         Input array. If `arr` is not an array, a conversion is attempted.
-    axis : {int, None}, optional
-        Axis along which the median is to be computed. The default (axis=None)
-        is to compute the mean of the flattened array.
+    axis : {int, None}
+        Axis along which the median is to be computed.
     
     Returns
     -------
@@ -340,7 +343,7 @@ def nanmedian_selector(arr, axis):
         if axis < 0:
             axis += ndim
     else:
-        a = a.ravel()
+        a = PyArray_Ravel(a, NPY_CORDER)
         axis = 0
         ndim = 1
     key = (ndim, dtype, axis)
