@@ -27,6 +27,9 @@ def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a):
 
 loop = {}
 loop[2] = """\
+    if nINDEX1 == 0:
+        msg = "numpy.nanmin raises on a.shape[axis]==0; so Bottleneck does."
+        raise ValueError(msg)
     for iINDEX0 in range(nINDEX0):
         amin = MAXDTYPE
         allnan = 1
@@ -42,6 +45,9 @@ loop[2] = """\
     return y
 """
 loop[3] = """\
+    if nINDEX2 == 0:
+        msg = "numpy.nanmin raises on a.shape[axis]==0; so Bottleneck does." 
+        raise ValueError(msg)
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
             amin = MAXDTYPE
@@ -67,6 +73,9 @@ floats_None['axisNone'] = True
 
 loop = {}
 loop[1] = """\
+    if nINDEX0 == 0:
+        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too." 
+        raise ValueError(m)
     amin = MAXDTYPE
     for iINDEX0 in range(nINDEX0):
         ai = a[INDEXALL]
@@ -76,9 +85,12 @@ loop[1] = """\
     if allnan == 0:       
         return np.DTYPE(amin)
     else:
-        return NAN
+        return np.DTYPE(NAN)
 """
 loop[2] = """\
+    if nINDEX0 * nINDEX1 == 0:
+        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too." 
+        raise ValueError(m)
     amin = MAXDTYPE
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
@@ -89,9 +101,12 @@ loop[2] = """\
     if allnan == 0:       
         return np.DTYPE(amin)
     else:
-        return NAN
+        return np.DTYPE(NAN)
 """
 loop[3] = """\
+    if nINDEX0 * nINDEX1 * nINDEX2 == 0:
+        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too." 
+        raise ValueError(m)
     amin = MAXDTYPE
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
@@ -103,7 +118,7 @@ loop[3] = """\
     if allnan == 0:       
         return np.DTYPE(amin)
     else:
-        return NAN
+        return np.DTYPE(NAN)
 """
 
 floats_None['loop'] = loop
@@ -115,6 +130,9 @@ ints['dtypes'] = INT_DTYPES
 
 loop = {}
 loop[2] = """\
+    if nINDEX1 == 0:
+        msg = "numpy.nanmin raises on a.shape[axis]==0; so Bottleneck does."
+        raise ValueError(msg)
     for iINDEX0 in range(nINDEX0):
         amin = MAXDTYPE
         for iINDEX1 in range(nINDEX1):
@@ -125,6 +143,9 @@ loop[2] = """\
     return y
 """
 loop[3] = """\
+    if nINDEX2 == 0:
+        msg = "numpy.nanmin raises on a.shape[axis]==0; so Bottleneck does."
+        raise ValueError(msg)
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
             amin = MAXDTYPE
@@ -145,6 +166,9 @@ ints_None['axisNone'] = True
 
 loop = {}
 loop[1] = """\
+    if nINDEX0 == 0:
+        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too."
+        raise ValueError(m)
     amin = MAXDTYPE
     for iINDEX0 in range(nINDEX0):
         ai = a[INDEXALL]
@@ -153,6 +177,9 @@ loop[1] = """\
     return np.DTYPE(amin)
 """
 loop[2] = """\
+    if nINDEX0 * nINDEX1 == 0:
+        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too."
+        raise ValueError(m)
     amin = MAXDTYPE
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
@@ -162,6 +189,9 @@ loop[2] = """\
     return np.DTYPE(amin)
 """
 loop[3] = """\
+    if nINDEX0 * nINDEX1 * nINDEX2 == 0:
+        m = "numpy.nanmin raises on a.size==0 and axis=None; Bottleneck too."
+        raise ValueError(m)
     amin = MAXDTYPE
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
@@ -193,7 +223,7 @@ nanmin['templates']['float'] = floats
 nanmin['templates']['float_None'] = floats_None
 nanmin['templates']['int'] = ints
 nanmin['templates']['int_None'] = ints_None
-nanmin['pyx_file'] = 'func/nanmin.pyx'
+nanmin['pyx_file'] = 'func/%sbit/nanmin.pyx'
 
 nanmin['main'] = '''"nanmin auto-generated from template"
 
@@ -255,9 +285,8 @@ def nanmin_selector(arr, axis):
     ----------
     arr : array_like
         Input array. If `arr` is not an array, a conversion is attempted.
-    axis : {int, None}, optional
-        Axis along which the minimum is to be computed. The default
-        (axis=None) is to compute the minimum of the flattened array.
+    axis : {int, None}
+        Axis along which the minimum is to be computed.
     
     Returns
     -------
@@ -295,10 +324,6 @@ def nanmin_selector(arr, axis):
         a = np.array(arr, copy=False)
     cdef int ndim = PyArray_NDIM(a)
     cdef int dtype = PyArray_TYPE(a)
-    cdef int size = PyArray_SIZE(a)
-    if size == 0:
-        msg = "numpy.nanmin() raises on size=0 input; so Bottleneck does too." 
-        raise ValueError, msg
     if (axis < 0) and (axis is not None):
         axis += ndim
     cdef tuple key = (ndim, dtype, axis)
