@@ -20,7 +20,7 @@ floats['top'] = """
 @cython.boundscheck(False)
 @cython.wraparound(False)
 def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a, int ddof):
-    "Variance of NDIMd array with dtype=DTYPE along axis=0 ignoring NaNs."
+    "Variance of NDIMd array with dtype=DTYPE along axis=AXIS ignoring NaNs."
     cdef int count = 0
     cdef np.DTYPE_t asum = 0, amean, ai
 """
@@ -148,38 +148,44 @@ ints['top'] = """
 @cython.boundscheck(False)
 @cython.wraparound(False)
 def NAME_NDIMd_DTYPE_axisAXIS(np.ndarray[np.DTYPE_t, ndim=NDIM] a, int ddof):
-    "Valriance of NDIMd array with dtype=DTYPE along axis=0 ignoring NaNs."
+    "Valriance of NDIMd array with dtype=DTYPE along axis=AXIS ignoring NaNs."
     cdef np.float64_t asum = 0, amean, ai
 """
 
 loop = {}
 loop[2] = """\
-    for iINDEX0 in range(nINDEX0):
-        asum = 0
-        for iINDEX1 in range(nINDEX1):
-            asum += a[INDEXALL]
-        amean = asum / nINDEX1
-        asum = 0
-        for iINDEX1 in range(nINDEX1):
-            ai = a[INDEXALL]
-            ai -= amean
-            asum += (ai * ai)
-        y[INDEXPOP] = sqrt(asum / (nINDEX1 - ddof))
-    return y 
-"""
-loop[3] = """\
-    for iINDEX0 in range(nINDEX0):
-        for iINDEX1 in range(nINDEX1):
+    if nINDEX1 == 0:
+        PyArray_FillWithScalar(y, NAN)
+    else:        
+        for iINDEX0 in range(nINDEX0):
             asum = 0
-            for iINDEX2 in range(nINDEX2):
+            for iINDEX1 in range(nINDEX1):
                 asum += a[INDEXALL]
-            amean = asum / nINDEX2
+            amean = asum / nINDEX1
             asum = 0
-            for iINDEX2 in range(nINDEX2):
+            for iINDEX1 in range(nINDEX1):
                 ai = a[INDEXALL]
                 ai -= amean
                 asum += (ai * ai)
-            y[INDEXPOP] = sqrt(asum / (nINDEX2 - ddof))
+            y[INDEXPOP] = sqrt(asum / (nINDEX1 - ddof))
+    return y 
+"""
+loop[3] = """\
+    if nINDEX2 == 0:
+        PyArray_FillWithScalar(y, NAN)
+    else:            
+        for iINDEX0 in range(nINDEX0):
+            for iINDEX1 in range(nINDEX1):
+                asum = 0
+                for iINDEX2 in range(nINDEX2):
+                    asum += a[INDEXALL]
+                amean = asum / nINDEX2
+                asum = 0
+                for iINDEX2 in range(nINDEX2):
+                    ai = a[INDEXALL]
+                    ai -= amean
+                    asum += (ai * ai)
+                y[INDEXPOP] = sqrt(asum / (nINDEX2 - ddof))
     return y 
 """
 ints['loop'] = loop
@@ -192,7 +198,9 @@ ints_None['axisNone'] = True
 
 loop = {}
 loop[1] = """\
-    size = nINDEX0    
+    size = nINDEX0
+    if size == 0:
+        return np.float64(NAN)
     for iINDEX0 in range(nINDEX0):
         ai = a[INDEXALL]
         if ai == ai:
@@ -204,10 +212,15 @@ loop[1] = """\
         if ai == ai:
             ai -= amean
             asum += (ai * ai)
-    return np.float64(sqrt(asum / (size - ddof)))
+    if size > ddof:        
+        return np.float64(sqrt(asum / (size - ddof)))
+    else:
+        return np.float64(NAN)
 """
 loop[2] = """\
     size = nINDEX0 * nINDEX1
+    if size == 0:
+        return np.float64(NAN)
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
             asum += a[INDEXALL]
@@ -218,10 +231,15 @@ loop[2] = """\
             ai = a[INDEXALL]
             ai -= amean
             asum += (ai * ai)
-    return np.float64(sqrt(asum / (size - ddof)))
+    if size > ddof:        
+        return np.float64(sqrt(asum / (size - ddof)))
+    else:
+        return np.float64(NAN)
 """
 loop[3] = """\
     size = nINDEX0 * nINDEX1 * nINDEX2
+    if size == 0:
+        return np.float64(NAN)
     for iINDEX0 in range(nINDEX0):
         for iINDEX1 in range(nINDEX1):
             for iINDEX2 in range(nINDEX2):
@@ -234,7 +252,10 @@ loop[3] = """\
                 ai = a[INDEXALL]
                 ai -= amean
                 asum += (ai * ai)
-    return np.float64(sqrt(asum / (size - ddof))) 
+    if size > ddof:        
+        return np.float64(sqrt(asum / (size - ddof)))
+    else:
+        return np.float64(NAN)
 """
 ints_None['loop'] = loop
 
@@ -257,7 +278,7 @@ nanstd['templates']['float'] = floats
 nanstd['templates']['float_None'] = floats_None
 nanstd['templates']['int'] = ints
 nanstd['templates']['int_None'] = ints_None
-nanstd['pyx_file'] = 'func/nanstd.pyx'
+nanstd['pyx_file'] = 'func/%sbit/nanstd.pyx'
 
 nanstd['main'] = '''"nanstd auto-generated from template"
 
@@ -350,10 +371,8 @@ def nanstd_selector(arr, axis):
     ----------
     arr : array_like
         Input array. If `arr` is not an array, a conversion is attempted.
-    axis : {int, None}, optional
-        Axis along which the standard deviation is to be computed. The
-        default (axis=None) is to compute the standard deviation of the
-        flattened array.
+    axis : {int, None}
+        Axis along which the standard deviation is to be computed.
     
     Returns
     -------
