@@ -1,17 +1,10 @@
 "Turn templates into Cython pyx files."
 
-import numpy as np
 
-if np.int_ == np.int32:
-    NPINT = 'int32'
-elif np.int_ == np.int64:
-    NPINT = 'int64'
-else:
-    raise RuntimeError('Expecting default NumPy int to be 32 or 64 bit.')
-
-def template(func):
+def template(func, bits):
     "Convert template dictionary `func` to a pyx file."
     codes = []
+    codes.append("# %s bit version\n" % str(bits))
     codes.append(func['main'])
     select = Selector(func['name'])
     for key in func['templates']:
@@ -25,7 +18,8 @@ def template(func):
                            reuse_non_nan_func=f['reuse_non_nan_func'],
                            is_reducing_function=func['is_reducing_function'],
                            cdef_output=func['cdef_output'],
-                           select=select)
+                           select=select,
+                           bits=bits)
         codes.append(code)    
     codes.append('\n' + str(select))
     if 'slow' in func:
@@ -35,14 +29,15 @@ def template(func):
             code2 = slow_functions(slow['name'],
                                    slow['signature'],
                                    slow['func'])
-            codes.append(code1)
             codes.append(code2)
-    fid = open(func['pyx_file'], 'w')
+            codes.append(code1)
+    fid = open(func['pyx_file'] % str(bits), 'w')
     fid.write(''.join(codes))
     fid.close()
 
 def subtemplate(name, top, loop, axisNone, dtypes, force_output_dtype,
-                reuse_non_nan_func, is_reducing_function, cdef_output, select):
+                reuse_non_nan_func, is_reducing_function, cdef_output, select,
+                bits):
     "Assemble template"
     ndims = loop.keys()
     ndims.sort()
@@ -78,7 +73,6 @@ def subtemplate(name, top, loop, axisNone, dtypes, force_output_dtype,
                     func = func.replace('NDIM', str(ndim))
                     func = func.replace('DTYPE', dtype)
                     func = func.replace('AXIS', str(axis))
-                    func = func.replace('NPINT', NPINT)
 
                     funcs.append(func)
                     select.append(ndim, dtype, axis)
@@ -277,9 +271,11 @@ def loop_cdef(ndim, dtype, axis, is_reducing_function, cdef_output=True):
 
     >>> print loop_cdef(ndim, dtype, axis, is_reducing_function)
         cdef Py_ssize_t i0, i1, i2
-        cdef int n0 = a.shape[0]
-        cdef int n1 = a.shape[1]
-        cdef int n2 = a.shape[2]
+        cdef np.npy_intp *dim
+        dim = PyArray_DIMS(a)
+        cdef int n0 = dim[0]
+        cdef int n1 = dim[1]
+        cdef int n2 = dim[2]
         cdef np.npy_intp *dims = [n0, n2]
         cdef np.ndarray[np.float64_t, ndim=2] y = PyArray_EMPTY(2, dims,
                                                   NPY_float64, 0)
@@ -289,9 +285,11 @@ def loop_cdef(ndim, dtype, axis, is_reducing_function, cdef_output=True):
     >>> is_reducing_function = False     
     >>> print loop_cdef(ndim, dtype, axis, is_reducing_function)
         cdef Py_ssize_t i0, i1, i2
-        cdef int n0 = a.shape[0]
-        cdef int n1 = a.shape[1]
-        cdef int n2 = a.shape[2]
+        cdef np.npy_intp *dim
+        dim = PyArray_DIMS(a)
+        cdef int n0 = dim[0]
+        cdef int n1 = dim[1]
+        cdef int n2 = dim[2]
         cdef np.npy_intp *dims = [n0, n1, n2]
         cdef np.ndarray[np.float64_t, ndim=3] y = PyArray_EMPTY(3, dims,
                                                   NPY_float64, 0)
@@ -313,8 +311,10 @@ def loop_cdef(ndim, dtype, axis, is_reducing_function, cdef_output=True):
     cdefs.append(tab + 'cdef Py_ssize_t ' + idx)
     
     # Length along each dimension
+    cdefs.append(tab + "cdef np.npy_intp *dim")
+    cdefs.append(tab + "dim = PyArray_DIMS(a)")
     for dim in range(ndim):
-        cdefs.append(tab + "cdef int n%d = a.shape[%d]" % (dim, dim))
+        cdefs.append(tab + "cdef int n%d = dim[%d]" % (dim, dim))
     
     if not cdef_output:
         return '\n'.join(cdefs) + '\n'
@@ -377,7 +377,6 @@ def slow_selector(name, maxaxis=32):
     for axis in axes:
         tup = 2 * (name, str(axis)) 
         src.append(fmt % tup)
-    src.append('\n')
     return '\n'.join(src)
 
 def slow_functions(name, signature, func, maxaxis=32):
@@ -387,7 +386,7 @@ def slow_functions(name, signature, func, maxaxis=32):
     sig = "def %s_slow_axis%s(%s):"
     doc = '%s"Unaccelerated (slow) %s along axis %s."'
     function = "%sreturn %s\n"
-    src = []
+    src = ['\n']
     for axis in axes:
         
         axis = str(axis)
