@@ -1,5 +1,8 @@
 "Test functions."
 
+# For support of python 2.5
+from __future__ import with_statement
+
 import numpy as np
 from numpy.testing import (assert_equal, assert_array_equal, assert_raises,
                            assert_array_almost_equal)
@@ -10,6 +13,7 @@ import bottleneck as bn
 def arrays(dtypes=bn.dtypes, nans=True):
     "Iterator that yield arrays to use for unit testing."
     ss = {}
+    ss[0] = {'size':  0, 'shapes': [(0,), (0,0), (2,0), (2,0,1)]}
     ss[1] = {'size':  4, 'shapes': [(4,)]}
     ss[2] = {'size':  6, 'shapes': [(1,6), (2,3)]}
     ss[3] = {'size':  6, 'shapes': [(1,2,3)]}
@@ -33,22 +37,31 @@ def arrays(dtypes=bn.dtypes, nans=True):
                     a.flat[i] = np.inf
                     yield a
                     yield -a
+    if nans:
+        # nanmedian regression tests
+        a = np.array([1, nan, nan, 2])
+        yield a
+        a = np.vstack((a, a))
+        yield a
+        yield a.reshape(1,2,4)
 
 def unit_maker(func, func0, decimal=np.inf, nans=True):
-    "Test that bn.xxx gives the same output as np.."
+    "Test that bn.xxx gives the same output as np.xxx."
     msg = '\nfunc %s | input %s (%s) | shape %s | axis %s\n'
     msg += '\nInput array:\n%s\n'
     for i, arr in enumerate(arrays(nans=nans)):
         for axis in range(-arr.ndim, arr.ndim) + [None]:
             with np.errstate(invalid='ignore'):
+                actual = 'Crashed'
+                desired = 'Crashed'
                 actualraised = False
                 try:
-                    actual = func(arr, axis=axis)
+                    actual = func(arr.copy(), axis=axis)
                 except:
                     actualraised = True
                 desiredraised = False
                 try:
-                    desired = func0(arr, axis=axis)
+                    desired = func0(arr.copy(), axis=axis)
                 except:
                     desiredraised = True
             if actualraised and desiredraised:
@@ -105,7 +118,19 @@ def test_median():
 
 def test_nanmedian():
     "Test nanmedian."
-    yield unit_maker, bn.nanmedian, bn.slow.nanmedian, np.inf, False
+    yield unit_maker, bn.nanmedian, bn.slow.nanmedian
+   
+def test_rankdata():
+    "Test rankdata."
+    yield unit_maker, bn.rankdata, bn.slow.rankdata
+
+def test_nanrankdata():
+    "Test nanrankdata."
+    yield unit_maker, bn.nanrankdata, bn.slow.nanrankdata
+
+def test_ss():
+    "Test ss."
+    yield unit_maker, bn.ss, bn.slow.ss
 
 # ---------------------------------------------------------------------------
 # Check that exceptions are raised
